/*----------------------------------------------------------------------------
 *  var.c   - handling of variables
 *
 *  Copyright (c) 2001 Frank Meyer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Creation:       12.08.2001  fm
 *  Last Update:    $Id: var.c,v 1.2.2.1 2002/04/05 10:24:21 fm Exp $
 *----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "var.h"
#include "check.h"

#ifndef TRUE
# define TRUE               1
# define FALSE              0
#endif

#define OK		    0
#define ERR		    (-1)

#define			    COMMENT_COLUMN 33

typedef struct variables
{
    char *  package;
    char *  name;
    char *  content;
    char *  comment;
} VARIABLES;

static VARIABLES	variables[MAX_VARIABLES];
static int		variables_used = 0;

static int  parse_line (char *, char *, char *, int);
static int  set_variable (char *, char *, char *, char *);

extern FILE *	logfp;
extern void	log_perror (char *);
extern void	fatal_exit (char *);

/*------------------------------------------------------------------------------
 * strsave (s)                      - save string s
 *------------------------------------------------------------------------------
 */
char *
strsave (char * s)
{
    char *  t;

    if (s)
    {
	 t = malloc ((unsigned) strlen (s) + 1);

	 if (t)
	 {
	    (void) strcpy (t, s);
	 }
    }
    else
    {
	t = (char *) NULL;
    }

    return (t);
} /* strsave (s) */

/*----------------------------------------------------------------------------
 *  dump_variables (char * fname)
 *----------------------------------------------------------------------------
 */
int
dump_variables (char * fname)
{
    FILE *  fp;
    int	    i;

    fp = fopen (fname, "wb");

    if (! fp)
    {
	fatal_exit (fname);
	return (ERR);
    }

    for (i = 0; i < variables_used; i++)
    {
	fprintf (fp, "%s='%s'\n", variables[i].name, variables[i].content);
    }

    fclose (fp);
    return (OK);
} /* dump_variables (char * fname) */


/*----------------------------------------------------------------------------
 *  get_variable (char * name)
 *----------------------------------------------------------------------------
 */
char *
get_variable (char * name)
{
    int i;

    for (i = 0; i < variables_used; i++)
    {
	if (! strcmp (name, variables[i].name))
	{
	    return (variables[i].content);
	}
    }

    return ((char *) NULL);
} /* get_variable (name) */


/*----------------------------------------------------------------------------
 *  get_variable_comment (char * name)
 *----------------------------------------------------------------------------
 */
char *
get_variable_comment (char * name)
{
    int i;

    for (i = 0; i < variables_used; i++)
    {
	if (! strcmp (name, variables[i].name))
	{
	    return (variables[i].comment);
	}
    }

    return ((char *) NULL);
} /* get_variable_comment (name) */


/*----------------------------------------------------------------------------
 *  chkvar (char * name, int mode)
 *----------------------------------------------------------------------------
 */
int
chkvar (char * name, int mode)
{
    char *  p = get_variable (name);

    if (! p)
    {
	fprintf (stderr, "Error: variable '%s' not defined\n", name);
	fprintf (logfp, "Error: variable '%s' not defined\n", name);
	return (ERR);
    }

    if (mode != CHECK_NONE && mode != CHECK_EIPADDR)
    {
	if (! *p)
	{
	    fprintf (stderr, "Error: empty variable: %s\n", name);
	    fprintf (logfp, "Error: empty variable: %s\n", name);
	    return (ERR);
	}
    }

    /* not else ! */

    if (mode == CHECK_YESNO)
    {
	if (strcmp (p, "yes") && strcmp (p, "no"))
	{
	    fprintf (stderr, "Error: wrong value of variable %s: '%s'\n",
		     name, p);
	    fprintf (logfp, "Error: wrong value of variable %s: '%s'\n",
		     name, p);
	    return (ERR);
	}
    }
    else if (mode == CHECK_NUMERIC)
    {
	int v = atoi (p);

	if (v <= 0 && strcmp (p, "0"))
	{
	    fprintf (stderr, "Error: wrong value of variable %s: '%s'\n",
		     name, p);
	    fprintf (logfp, "Error: wrong value of variable %s: '%s'\n",
		     name, p);
	    return (ERR);
	}
    }
    else if (mode == CHECK_IPADDR)
    {
	int a1, a2, a3, a4;

	if (sscanf (p, "%d.%d.%d.%d", &a1, &a2, &a3, &a4) != 4 ||
	    a1 < 0 || a1 > 255 || a2 < 0 || a2 > 255 ||
	    a3 < 0 || a3 > 255 || a4 < 0 || a4 > 255)
	{
	    fprintf (stderr,
	     "Error: no valid ip address in variable %s: '%s'\n", name, p);
	    fprintf (logfp,
	     "Error: no valid ip address in variable %s: '%s'\n", name, p);
	    return (ERR);
	}
    }
    else if (mode == CHECK_EIPADDR)
    {
	if (*p)
	{
	    int a1, a2, a3, a4;

	    if (sscanf (p, "%d.%d.%d.%d", &a1, &a2, &a3, &a4) != 4 ||
		a1 < 0 || a1 > 255 || a2 < 0 || a2 > 255 ||
		a3 < 0 || a3 > 255 || a4 < 0 || a4 > 255)
	    {
		fprintf (stderr,
		 "Error: no valid ip address in variable %s: '%s'\n", name, p);
		fprintf (logfp,
		 "Error: no valid ip address in variable %s: '%s'\n", name, p);
		return (ERR);
	    }
	}
    }
    else if (mode == CHECK_NOBLANK)
    {
	if (strchr (p, ' '))
	{
	    fprintf (stderr,
	     "Error: variable '%s' may not contain a space: '%s'\n", name, p);
	    fprintf (logfp,
	     "Error: variable '%s' may not contain a space: '%s'\n", name, p);
	    return (ERR);
	}
    }
    else if (mode == CHECK_DIALMODE)
    {
	if (strcmp (p, "auto") && strcmp (p, "manual") && strcmp (p, "off"))
	{
	    fprintf (stderr,
	    "Error: allowed values of variable '%s': 'auto', 'manual', 'off'\n",
	     name);
	    fprintf (logfp,
	    "Error: allowed values of variable '%s': 'auto', 'manual', 'off'\n",
	     name);
	    return (ERR);
	}
    }

    return (OK);
} /* chkvar (name, mode) */


/*----------------------------------------------------------------------------
 *  chkvar_idx (char * fmt, char * max_idx_name, int mode)
 *----------------------------------------------------------------------------
 */
int
chkvar_idx (char * fmt, char * max_idx_name, int mode)
{
    char    name[64];
    char    new_fmt[64];
    char *  p;
    int	    max_idx;
    int	    idx;

    for (p = new_fmt; *fmt; fmt++)
    {
	*p++ = *fmt;

	if (*fmt == '%')
	{
	    *p++ = 'd';
	}
    }

    *p = '\0';

    p = get_variable (max_idx_name);

    if (! p)
    {
	fprintf (stderr, "Error: variable '%s' not defined\n", max_idx_name);
	fprintf (logfp, "Error: variable '%s' not defined\n", max_idx_name);
	return (ERR);
    }

    max_idx = atoi (p);

    for (idx = 1; idx <= max_idx; idx++)
    {
	sprintf (name, new_fmt, idx);

	if (chkvar (name, mode) != OK)
	{
	    return (ERR);
	}
    }

    return (OK);
} /* chkvar_idx (fmt, max_idx, mode) */

int
read_config_file (char * config_file, char * package)
{
    char        buf[1024];
    char *	p;
    FILE *      config_fp;
    int         line;

    config_fp = fopen (config_file, "r");

    if (! config_fp)
    {
	log_perror (config_file);
	return (ERR);
    }

    for (line = 1; fgets (buf, 1024, config_fp); line++)
    {
	if (buf[0] == '#' && buf[1] == '#')	/* double comment: '##'	    */
	{					/* don't copy it!	    */
	    continue;
	}

	p = strchr (buf, '\r');			/* unix: we read a DOS file */

	if (p)
	{
	    *p = '\0';
	}
	else
	{
	    p = strchr (buf, '\n');

	    if (p)
	    {
		*p = '\0';
	    }
	}

	for (p = buf; *p; p++)
	{
	    if (*p != ' ' && *p != '\t')
	    {
		break;
	    }
	}

	if (*p != '\0' && *p != '#')
	{
	    if (parse_line (config_file, package, buf, line) != OK)
	    {
		return (ERR);
	    }
	}

	line++;
    }

    fclose (config_fp);

    return (OK);
} /* read_config_file (char * config_file) */

/*----------------------------------------------------------------------------
 *  parse_line (char * config_file, char * package, char * buf, int line)
 *----------------------------------------------------------------------------
 */
static int
parse_line (char * config_file, char * package, char * buf, int line)
{
    char *	    variable;
    char *	    content;
    char *	    comment;
    char *	    p;
    char *	    pp;
    int		    rtc;

    if (*buf == '=')
    {
	fprintf (stderr, "%s: line %d: syntax error: no variable name\n",
		 config_file, line);
	fprintf (logfp, "%s: line %d: syntax error: no variable name\n",
		 config_file, line);
	return (ERR);
    }

    p = strchr (buf, '=');

    if (! p)
    {
	fprintf (stderr, "%s: line %d: syntax error: no '=' found\n",
		 config_file, line);
	fprintf (logfp, "%s: line %d: syntax error: no '=' found\n",
		 config_file, line);
	return (ERR);
    }

    pp = p - 1;

    while (pp != buf && (*pp == ' ' || *pp == '\t'))
    {
	*pp-- = '\0';
    }

    *p++ = '\0';

    while (*p == ' ' || *p == '\t')		/* skip spaces after '='    */
    {
	p++;
    }

    if (*p != '\'')
    {
	fprintf (stderr, "%s: line %d: syntax error, missing \"'\"\n",
		 config_file, line);
	fprintf (logfp, "%s: line %d: syntax error, missing \"'\"\n",
		 config_file, line);
	return (ERR);
    }

    content = p + 1;

    p = strchr (content, '\'');

    if (! p)
    {
	fprintf (stderr, "%s: line %d: syntax error, missing \"'\"\n",
		 config_file, line);
	fprintf (logfp, "%s: line %d: syntax error, missing \"'\"\n",
		 config_file, line);
	return (ERR);
    }

    *p = '\0';

    comment = strchr (p + 1, '#');

    if (comment)
    {
	comment = comment + 1;
    }

    variable = buf;

    if (! *variable)
    {
	fprintf (stderr, "%s: line %d: syntax error: no variable name\n",
		 config_file, line);
	fprintf (logfp, "%s: line %d: syntax error: no variable name\n",
		 config_file, line);
	return (ERR);
    }

    rtc = set_variable (package, variable, content, comment);

    if (rtc == ERR)
    {
	fprintf (stderr, "%s: line %d: variable %s already defined\n",
		 config_file, line, variable);
	fprintf (logfp, "%s: line %d: variable %s already defined\n",
		 config_file, line, variable);
	return (ERR);
    }

    return (rtc);
} /* parse_line (char * config_file, char * buf, int line) */

/*----------------------------------------------------------------------------
 *  set_variable (char * name, char * content, comment)
 *----------------------------------------------------------------------------
 */
static int
set_variable (char * package, char * name, char * content, char * comment)
{
    int i;

    for (i = 0; i < variables_used; i++)
    {
	if (! strcmp (name, variables[i].name))
	{
	    return (ERR);
	}
    }

    if (variables_used < MAX_VARIABLES - 1)
    {
	variables[variables_used].package   = strsave (package);
	variables[variables_used].name      = strsave (name);
	variables[variables_used].content   = strsave (content);

	if (comment)
	{
	    variables[variables_used].comment = strsave (comment);
	}
	else
	{
	    variables[variables_used].comment = (char *) NULL;
	}

	variables_used++;
    }
    else
    {
	fprintf (stderr, "too many variables, max is %d.\n", MAX_VARIABLES);
	fprintf (logfp, "too many variables, max is %d.\n", MAX_VARIABLES);
	return (FATAL_ERR);
    }

    return (OK);
} /* set_variable (package, name, content, comment) */

/*----------------------------------------------------------------------------
 *  set_variable_content (char * name, char * content)
 *----------------------------------------------------------------------------
 */
int
set_variable_content (char * name, char * content)
{
    int i;

    for (i = 0; i < variables_used; i++)
    {
	if (! strcmp (name, variables[i].name))
	{
	    if (variables[i].content)
	    {
		free (variables[i].content);
	    }

	    variables[i].content   = strsave (content);
	    return (OK);
	}
    }

    return (ERR);
} /* set_variable_content (char * name, char * content) */
